﻿using System.Drawing;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace Infragistics.Test
{
  class OverflowIndicatorFilter : IUIElementDrawFilter
  {
    public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
    {
      if (drawParams.Element is CellUIElement)
      {
        return DrawPhase.BeforeDrawImage;
      }
      return DrawPhase.None;
    }

    public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
    {
      switch(drawPhase)
      {
        case DrawPhase.BeforeDrawImage:
          var cell = drawParams.Element.SelectableItem as UltraGridCell;

          if (cell == null) return true;
          if (!ShouldDraw(cell, drawParams.Graphics, drawParams.Font)) return true;

          DrawTriangle(drawParams.Graphics, drawParams.Element.Rect, IsLastCell(cell));

          return true;
        default:
          return false;
      }
    }

    /// <summary>
    /// Measure the contents of the cell. If the text is bigger than the cell width, draw the triangle. 
    /// </summary>
    private static bool ShouldDraw (UltraGridCell cell_, Graphics graphics_, Font font_)
    {
      var size = graphics_.MeasureString(cell_.Text, font_);

      return (size.Width > cell_.Width);
    }

    /// <summary>
    /// Need to check if it's the last cell in the row because it needs one less
    /// pixel of triangle. No cell border, just row border? 
    /// </summary>
    private static bool IsLastCell (UltraGridCell cell_)
    {
      int n = cell_.Row.Cells.Count;
      int i = cell_.Row.Cells.IndexOf(cell_);

      return n == i + 1;
    }

    private static void DrawTriangle (Graphics g_, Rectangle rect_, bool last_)
    {
      int triangleSize = last_ ? 7 : 8; // if it's in the last row, need a smaller triangle

      var corner = new Point(rect_.Location.X + rect_.Width, rect_.Location.Y + rect_.Height);
      var vertical = new Point(corner.X, corner.Y - triangleSize);
      var horiz = new Point(corner.X - triangleSize, corner.Y);

      g_.FillPolygon(Brushes.Black, new[] { corner, vertical, horiz });
    }
  }
}
